<?php

namespace Drupal\search_replace_aliases\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\path_alias\Entity\PathAlias;

/**
 * Implements the Search Replace Aliases form.
 */
class SearchReplaceAliasesForm extends FormBase
{

    /**
     * Defines the form ID.
     */
    public function getFormId()
    {
        return 'search_replace_aliases_form';
    }

    /**
     * Builds the form.
     */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
        // Warning message about redirection.
        $form['redirect_warning'] = [
        '#markup' => $this->t('<strong>Warning!</strong> If you wish to create a redirection, make sure the <em>Redirect</em> module is properly installed and configured.'),
        '#prefix' => '<div class="warning-message">',
        '#suffix' => '</div>',
        ];

        // Form fields for search and replace strings.
        $form['search_string'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Search Alias'),
        '#description' => $this->t('Enter the fragment of the alias you wish to search for.'),
        '#required' => true,
        ];

        $form['replace_string'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Replace With'),
        '#description' => $this->t('Enter the string you wish to replace with.'),
        '#required' => true,
        ];

        // Hidden field to track the preview step.
        $form['preview_step'] = [
        '#type' => 'hidden',
        '#value' => $form_state->get('preview_step') ? $form_state->get('preview_step') : '0',
        ];

        // Adjust the submit button based on the preview step.
        if ($form_state->get('preview_step') == '1') {
            // Only show the button after the preview list.
            $aliases_to_change = $form_state->get('aliases_to_change');

            // Message indicating how many aliases will be modified.
            if ($aliases_to_change) {
                $count_aliases = count($aliases_to_change);
                $form['count_message'] = [
                '#markup' => $this->t('<strong>@count alias(es) will be modified:</strong>', ['@count' => $count_aliases]),
                '#prefix' => '<div class="count-message">',
                '#suffix' => '</div>',
                ];

                // Show the list of changes.
                $list_items = [];
                foreach ($aliases_to_change as $alias_change) {
                    $list_items[] = $this->t(
                        '<li><s>@old</s> <strong>@new</strong></li>', [
                        '@old' => $alias_change['old'],
                        '@new' => $alias_change['new'],
                        ]
                    );
                }
                $form['preview_list'] = [
                '#markup' => '<ul>' . implode('', $list_items) . '</ul>',
                ];
            }

            // Now add the apply changes button.
            $form['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Confirm and Apply Changes'),
            ];
        } else {
            $form['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Preview Changes'),
            ];
        }

        return $form;
    }

    /**
     * Form submission handler.
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        // Get form values.
        $search_string = $form_state->getValue('search_string');
        $replace_string = $form_state->getValue('replace_string');

        // Search all aliases containing the search string.
        $query = \Drupal::entityQuery('path_alias')
            ->condition('alias', '%' . $search_string . '%', 'LIKE')
            ->accessCheck(false);
        $results = $query->execute();

        if ($form_state->get('preview_step') == '1') {
            // If we are in the preview step, apply the changes.
            $this->startBatchProcess($results, $search_string, $replace_string);
        } else {
            // Preview step.
            if (!empty($results)) {
                $aliases_to_change = $this->getAliasChanges($results, $search_string, $replace_string);

                // Set the preview step to true and store the aliases to change in form_state.
                $form_state->set('preview_step', '1');
                $form_state->set('aliases_to_change', $aliases_to_change);
                $form_state->setRebuild(true);
            } else {
                \Drupal::messenger()->addMessage($this->t('No aliases found for the given search string.'), 'warning');
            }
        }
    }

    /**
     * Starts the batch process.
     */
    protected function startBatchProcess($results, $search_string, $replace_string)
    {
        $batch = [
        'title' => $this->t('Replacing aliases...'),
        'operations' => [],
        'finished' => [$this, 'batchFinished'],
        ];

        // Split the work into operations.
        foreach ($results as $alias_id) {
            $batch['operations'][] = [
            [$this, 'replaceAlias'],
            [$alias_id, $search_string, $replace_string],
            ];
        }

        // Start the batch.
        batch_set($batch);
    }

    /**
     * Alias replacement operation.
     */
    public function replaceAlias($alias_id, $search_string, $replace_string, array &$context)
    {
        // Load the alias from the database.
        $path_alias = PathAlias::load($alias_id);
        if ($path_alias) {
            $old_alias = $path_alias->getAlias();
            $new_alias = str_replace($search_string, $replace_string, $old_alias);
            $path_alias->setAlias($new_alias);
            $path_alias->save();
        }

        // Update the batch context.
        $context['results']['replaced'][] = $alias_id;
    }

    /**
     * Function called when the batch finishes.
     */
    public function batchFinished($success, array $results, array $operations)
    {
        if ($success) {
            $count = count($results['replaced']);
            \Drupal::messenger()->addMessage($this->t('@count alias(es) have been successfully replaced.', ['@count' => $count]));
        } else {
            \Drupal::messenger()->addMessage($this->t('There was a problem replacing the aliases.'), 'error');
        }
    }

    /**
     * Helper function to retrieve alias changes for preview.
     */
    protected function getAliasChanges($results, $search_string, $replace_string)
    {
        $aliases_to_change = [];
        foreach ($results as $alias_id) {
            $path_alias = PathAlias::load($alias_id);
            if ($path_alias) {
                $old_alias = $path_alias->getAlias();
                $new_alias = str_replace($search_string, $replace_string, $old_alias);
                $aliases_to_change[] = [
                'old' => $old_alias,
                'new' => $new_alias,
                ];
            }
        }
        return $aliases_to_change;
    }
}
